;*******************************************************************************************
;   Beschreibung:	Steuerung/berwachung fr Aktivbox								 	   * 
;		Oszillatorfrequenz: 4MHz																												 *							 *
;*******************************************************************************************
;                                                                                          *
;    Dateiname:	 	Aktivbox_P628A.asm						                       		   *
;    Datum:        	28.06.2017                                                             *
;    Version: 		V1.0                                                                   *
;                                                                                          *
;    Autor:       	Werner Hirsch                                                          *
;    Internet:      www.wernerhirsch.de	                                                   *
;    e-mail:		wernerhirschhh@aol.com                                                 *
;                                                                                          *
;*******************************************************************************************
;    Bemerkungen:                                                                          *
;    		Fr die Simulation der Portdaten kann man den Simulator von MPLAB verwenden:   *
;			Debugger->Select Tool->MPLAB SIM (Debugger auswhlen)						   *
;			Im Fenster Stimulus auf den Tab "Pin/Register Actions" umschalten              *
;																														 *
;	Bankumschaltungen:
;	BANK_0:	BCF STATUS,RP0
;			BCF STATUS,RP1
;	BANK_1:	BSF STATUS,RP0
;			BCF STATUS,RP1
;	BANK_2:	BCF STATUS,RP0
;			BSF STATUS,RP1
;	BANK_3:	BSF STATUS,RP0
;			BSF STATUS,RP1
;													 *
;                                                                                          *
;*******************************************************************************************


	list		p=16f628A			;definiert den verwendeten Prozessor
	#include	<p16f628A.inc>		;in diesem File stehen die Adressen und Bitnamen
	ERRORLEVEL	-302				;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL  -305				;SUPPRESS BANK SELECTION MESSAGES
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_OFF & _XT_OSC & _DATA_CP_OFF & _LVP_OFF & _MCLRE_ON 
	; Mit '__CONFIG' werden die Configurationsbits gesetzt, z.B. Watchdog an/aus, Debugmodus

;===========================
;  DEFINITIONEN
;===========================
#define OUT_STBY		PORTA, 0		;Pin RA0 (Pin 17)Ausgang Standby
#define OUT_MUTE		PORTA, 1		;Pin RA1 (Pin 18)Ausgang Mute und LED grn
#define OUT_POWER		PORTA, 2		;Pin RA2 (Pin 1) Ausgang Power
#define OUT_VOL_M		PORTA, 3		;Pin RA3 (Pin 2) Ausgang Vol- (fr ext. Vorverstrker) und LED gelb
#define IN_MUTE			PORTA, 4		;Pin RA4 (Pin 3) Eingang Freigabesignal ext. oder intern
#define IN_IR			PORTB, 0		;Pin RB0 (Pin 6) Eingang IR Signal
#define OUT_LED_RT		PORTB, 1		;Pin RB1 (Pin 7) Ausgang LED rot
#define IN_RESET		PORTB, 2		;Pin RB2 (Pin 8) Eingang Reset Taster und aktivieren/speichern bei "learn"
#define IN_DETECT		PORTB, 3		;Pin RB3 (Pin 9) Eingang Detect Signal
#define IN_IR_MODE		PORTB, 4		;Pin RB4 (Pin 10)Eingang fr IR-Mode
#define IN_NCB5			PORTB, 5		;Pin RB5 (Pin 11)Eingang NC
#define IN_NCB6			PORTB, 6		;Pin RB6 (Pin 12)PGC, Eingang NC 
#define IN_NCB7			PORTB, 7		;Pin RB7 (Pin 13)PGD, Eingang NC



;===========================
;  VARIABLEN
;===========================
w_temp        	EQU	0x70        ; werden fr Interrupts verwendet, um den Status vor 
status_temp   	EQU	0x71        ; der Interruptroutine zu sichern
pclath_temp		EQU	0x72

DET_LOOP		EQU 0x20        ;Zhler fr Loops gesamt fr Detect(zur Reduzierung des Volumens)
DET_NUM			EQU	0x21		;Register fr Anzahl Detect (zur Reduzierung des Volumens)
DET_NUM_MAX		EQU	0x22		;Register fr maximale Anzahl Detect (zur Reduzierung des Volumens)
CLIP_COUNT		EQU	0x23		;Register fr Anzahl Clip (zur Abschaltung des Verstrkers)
CLIP_COUNT_MAX	EQU	0x24		;Register fr maximale Anzahl Clip (zur Abschaltung des Verstrkers)
CLIP_1			EQU	0x25		;Speicherregister fr Clip Werte 1-8
CLIP_2			EQU	0x26		;Speicherregister fr Clip Werte 9-16
CLIP_BIT		EQU	0x27		;Register fr Zwischenspeicherung des Bit Wertes 0/1
STAT_AMP		EQU	0x28		;Status Register fr Verstrker: 0 = off, 1 = on
TEMP			EQU 0x29        ;Allgemeines Zwischenspeicherregister
TMR0_LOOP		EQU 0x2A        ;Timer 0 Schleifenregister

IR_START2		EQU 0x2B		;Register fr 2.Startbit
IR_STRG			EQU	0x2C		;Register fr Steuerbit
IR_ADR			EQU	0x2D		;Register fr die empfangene RC5 Adresse
IR_BEF			EQU	0x2E		;Register fr den empfangenen RC5 Befehl
IR_BIT			EQU 0x2F		;Status IR Bit: Bit0 = 1.Hlfte,  100 = Fehler, 001 = 0, 010 = 1, 111 = Fehler
								;				Bit1 = 2.Hlfte
								;				Bit2 = Fehlerbit
IR_TEMP			EQU	0x30		;Temp Register fr Zwischspeicherung des gesendeten bytes
IR_ADR_1		EQU	0x31		;Register fr hinterlegte RC5 Adresse
IR_BEF_1		EQU	0x32		;Register fr hinterlegten IR-Befehl (fr on/off)
IR_ADR_2		EQU	0x33		;Register fr hinterlegte RC5 Adresse
IR_BEF_2		EQU	0x34		;Register fr hinterlegten IR-Befehl (fr on/off)
IR_NEU			EQU 0x35		;Statusregister:	Bit0 = neuer IR-Befehl	Bit1 = Fehlerbit
COUNTER			EQU	0x36		;Allgemeines Zhlregister
LOOP_1			EQU 0x37		;Zhler1 fr Wartezeit
LOOP_2			EQU 0x38		;Zhler2 fr Wartezeit
LOOP_1us		EQU 0x39		;Zhler1 fr Wartezeit us
LOOP_2us		EQU 0x3A		;Zhler2 fr Wartezeit us

;__________________________________________________________________________________________
;
		ORG 	H'2100'			;Adresse des EEPROM fr den Brenner (2100 = erster Speicherplatz)
;Konstanten IR
		de		0x10			;Adresse 00,	RC5 Adresse 1, dez. = 16
		de		0x0C			;Adresse 01,	RC5 Befehl 2, dez. = 12
		de		0x10			;Adresse 02,	RC5 Adresse 1, dez. = 16
		de		0x0D			;Adresse 03,	RC5 Befehl 2, dez. = 13
;__________________________________________________________________________________________
;

		ORG     0x000			; Adresse, an die nach einem Reset gesprungen wird
		
  		goto    start           ; Springe zum Programmanfang

		ORG     0x004           ; Adresse, an die nach einem Interrupt gesprungen wird
		movwf	w_temp          ; Sichere den Inhalt des W-Registers
		movf    STATUS,w        ; Verschiebe STATUS-Register ins W-Register
		movwf	status_temp     ; Sichere den Inhalt des STATUS-Registers

	bcf INTCON,INTE				;Verbiete RB0 Interrupts
	call ir_startbit			;Aufruf des Unterprogramms zur Auswertung des IR Signals

		movf    status_temp,w  	; hole die Kopie des STATUS-Registers
		movwf	STATUS          ; Sichere die Daten wieder zurck in das STATUS-Register
		swapf   w_temp,f
		swapf   w_temp,w        ; Sichere die Daten wieder zurck in das W-Register
		retfie                  ; Rcksprung vom Interrupt
;__________________________________________________________________________________________

start							; Beginn des Programms
	; Initialisierungen
	bcf STATUS,RP0			;BANK_0
	bcf STATUS,RP1			;
	clrf PORTA				;Lsche alle Ausgangsports
	clrf PORTB
	movlw b'00000111'		;alle Pins von Port A sind digitale Ein- oder Ausgnge
	movwf CMCON
	bsf STATUS,RP0			;BANK_1
	movlw b'00001000'     	;pull-up on, Timer0 Prescaler 1:1
	movwf OPTION_REG 
	movlw b'11110000'		;setze RA0 - RA3 als Ausgang, den Rest als Eingang
	movwf TRISA
	movlw b'11111101'		;setze RB1 als Ausgang und den Rest als Eingang
	movwf TRISB
	bcf STATUS,RP0			;BANK_0
	clrf INTCON   			;	
	clrf TMR0

	bsf OUT_LED_RT			;rote LED an
	clrf DET_LOOP			;Lsche Register fr Loop detect
	clrf DET_NUM			;Lsche Register fr Anzahl detect
	clrf CLIP_COUNT			;Lsche Register
	clrf CLIP_1				;Lsche Register
	clrf CLIP_2				;Lsche Register
	clrf CLIP_BIT			;Lsche Register
	clrf STAT_AMP			;Lsche Register
	movlw d'4'
	movwf TMR0_LOOP
	movlw d'75'				;lade wert 75 ins W-Register (75 von 250 = 30%)
	movwf DET_NUM_MAX		;schiebe Wert ins F-Register	
	movlw d'8'				;lade wert 8 ins W-Register (8 von 16 = 50%)
	movwf CLIP_COUNT_MAX	;schiebe Wert ins F-Register	

	movlw d'0'				;lade EEPROM Adresse ins W-Register
	call readEE				;Aufruf des Unterprogramms zum Lesen des EEPROM, (IR-Adresse 1)
	movwf IR_ADR_1			;schiebe Wert ins F-Register	
	movlw d'1'				;lade EEPROM Adresse ins W-Register
	call readEE				;Aufruf des Unterprogramms zum Lesen des EEPROM, (IR-Befehl 1)
	movwf IR_BEF_1			;schiebe Wert ins F-Register
	movlw d'2'				;lade EEPROM Adresse ins W-Register
	call readEE				;Aufruf des Unterprogramms zum Lesen des EEPROM, (IR-Adresse 2)
	movwf IR_ADR_2			;schiebe Wert ins F-Register	
	movlw d'3'				;lade EEPROM Adresse ins W-Register
	call readEE				;Aufruf des Unterprogramms zum Lesen des EEPROM, (IR-Befehl 2)
	movwf IR_BEF_2			;schiebe Wert ins F-Register

	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife

	btfsc IN_RESET			;Abfrage ob Reset Taster gedrckt ist, (hier ob der Taster fr IR-learn gedrckt ist)
	goto $+6				;Taster ist nicht gedrckt, weiter im Start Programm
	bcf OUT_LED_RT			;Taster ist gedrckt, lsche rote LED
	bsf STAT_AMP,2			;Setze Statusbit fr Speicherung der ersten Adresse/Befehl
	btfss IN_RESET			;Prfen ob Taster noch gedrckt ist
	goto $-1				;Taster ist noch gedrckt, zurck zur Prfung
	goto learn				;Taster ist nicht mehr gedrckt, weiter bei IR learn

	clrf IR_NEU				;lsche Statusregister
	bcf INTCON,INTF			;lsche Interruptflag
	bsf INTCON,INTE			;erlaube RB0 interrupts
	bsf INTCON,GIE			;erlaube globale interrupts

;Testbereich Anfang
	nop
;Testbereich Ende

main						;Beginn der Hauptschleife
							;
	btfss IN_MUTE			;Prfen, ob der Eingang fr die Amp Freigabe high ist (=> freigegeben)
	goto test_in_mute_off	;nicht freigegeben, weiter zur Prfung Statusbit
	btfsc STAT_AMP,0		;freigegeben, Prfen des Statusbit
	goto test_in_reset_on	;Keine Vernderung, weiter im Programm
	bsf STAT_AMP,0			;Statusbit ist low, Verstrker soll eingeschaltet werden.
							;Setze Statusbit fr internes/externes Freigabesignal vorhanden
	call amp_on				;Aufruf des Unterprogramms fr Verstrker Einschalten
	goto test_in_reset_on	;weiter im Programm
test_in_mute_off
	btfss STAT_AMP,0		;nicht freigegeben, Prfen des Statusbit
	goto test_in_reset_on	;Keine Vernderung, weiter im Programm
	bcf STAT_AMP,0			;Lsche Statusbit fr internes/externes Freigabesignal nicht vorhanden
	call amp_off			;Aufruf des Unterprogramms fr Verstrker Ausschalten

test_in_reset_on
	btfsc IN_RESET			;Prfen, ob der Taster gedrckt ist (Ein/Ausschalten)
	goto test_ir			;nicht gedrckt, weiter im Programm
	btfsc STAT_AMP,1		;gedrckt, Prfen des Statusbit
	goto $+3				;Statusbit ist high, Verstrker soll ausgeschaltet werden.
	call amp_on				;Statusbit ist low, Aufruf des Unterprogramms fr Verstrker Einschalten
	goto test_ir			;Taster ist nicht mehr gedrckt, weiter im Programm
	call amp_off			;Statusbit ist high, Aufruf des Unterprogramms fr Verstrker Ausschalten

test_ir	
	btfss IR_NEU,0			;prfe Bit fr neuen IR-Befehl
	goto check_clip			;kein neuer IR-Befehl
	btfsc IR_NEU,1			;prfe IR-Fehler Bit
	goto auswert_ir_end		;Fehlerbit ist gesetzt
	
	movfw IR_ADR_1			;hole die, fr die Auswertung gltige 1.Adresse ins W-Register
	subwf IR_ADR,w			;subtrahiere die empfangene Adresse
	btfss STATUS,Z			;prfen ob Ergebnis =0
	goto ir_adr_2			;empfangene Adresse entspricht nicht der gespeicherten 1.Adresse => weiter bei 2.Adr
	movfw IR_BEF_1			;hole den, fr die Auswertung gltigen 1.Befehl ins W-Register
	subwf IR_BEF,w			;subtrahiere den empfangenen Befehl
	btfss STATUS,Z			;prfen ob Ergebnis =0
	goto ir_adr_2			;empfangener Befehl entspricht nicht dem gespeicherten 1.Befehl => weiter bei 2.Adr
	btfss IN_IR_MODE		;Abfrage, welcher Mode verwendet werden soll
	goto $+3				;ON Betrieb ist eingestellt, 
	call ir_onoff			;ON/OFF Betrieb ist eingestellt, Aufruf des Unterprogramms
	goto auswert_ir_end
	btfsc STAT_AMP,1
	goto auswert_ir_end
	call amp_on
	goto auswert_ir_end
ir_adr_2
	movfw IR_ADR_2			;hole die, fr die Auswertung gltige 2.Adresse ins W-Register
	subwf IR_ADR,w			;subtrahiere die empfangene Adresse
	btfss STATUS,Z			;prfen ob Ergebnis =0
	goto auswert_ir_end		;empfangene Adresse entspricht nicht der gespeicherten 2.Adresse => Abbruch
	movfw IR_BEF_2			;hole den, fr die Auswertung gltigen 2.Befehl ins W-Register
	subwf IR_BEF,w			;subtrahiere den empfangenen Befehl
	btfss STATUS,Z			;prfen ob Ergebnis =0
	goto auswert_ir_end		;empfangener Befehl entspricht nicht dem gespeicherten 2.Befehl => Abbruch
	btfss IN_IR_MODE		;Abfrage, welcher Mode verwendet werden soll
	goto $+3				;OFF Betrieb ist eingestellt, 
	call ir_onoff			;ON/OFF Betrieb ist eingestellt, Aufruf des Unterprogramms
	goto auswert_ir_end
	btfss STAT_AMP,1
	goto auswert_ir_end
	call amp_off
auswert_ir_end
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms 
	call wait				;Warteschleife
	clrf IR_NEU				;lsche Statusregister
	bcf INTCON,INTF			;lsche Interruptflag
	bsf INTCON,INTE			;erlaube RB0 interrupts

check_clip					;Abfrage des Detect Eingangs alle 1,024ms (4 x 0,256ms)
	btfss INTCON,T0IF		;Abfrage des Timer 0 berlaufs
	goto main				;Timer 0 noch nicht bergelaufen, zurck zur Hauptschleife
	bcf INTCON,T0IF			;Timer 0 bergelaufen
	decfsz TMR0_LOOP		;reduziere Zhler um 1 und prfe ob Zhler =0
	goto main				;Zhler noch nicht 0, zurck zur Hauptschleife
	movlw d'4'				;Zhler =0, d.h. 1,024ms sind abgelaufen, Rcksetzen des Zhlers mit 4
	movwf TMR0_LOOP

	btfss IN_DETECT			;Prfen, ob das Signal fr IN_DETECT high ist
	incf DET_NUM			;wenn nein, erhhe Zhler um 1

	decfsz DET_LOOP			;reduziere Zhler um 1 und prfe ob 0
	goto main				;Zhler ist noch nicht 0
	movlw d'250'			;Rcksetzen des Zhlers mit Wert 250
	movwf DET_LOOP

	movfw DET_NUM			;hole Zhler ins W-Register
	subwf DET_NUM_MAX,W		;subtrahiere max Anzahl
	btfsc STATUS,C			;wenn ergebnis negativ, C=0 nchste Befehl, wenn ergebnis positiv, C=1 berspringe nchsten Befehl
	goto detect_no			;Zhler < max,
	goto detect_yes			;Zhler > max,
detect_no
	bcf OUT_VOL_M			;setze Ausgang fr Vol- auf low
	clrf DET_NUM			;lschen des Zhlers
	clrf CLIP_BIT			;lsche Bit
	goto check_average
detect_yes
	bsf OUT_VOL_M			;setze Ausgang fr Vol- auf high
	clrf DET_NUM			;lschen des Zhlers
	bsf CLIP_BIT,0			;setze Bit

check_average
	rlf	CLIP_1,f			;schiebe das Speicherregister 1 um 1 nach links
	rlf CLIP_2,f			;schiebe das Speicherregister 2 um 1 nach links
	bcf CLIP_1,0			;lsche das eingeschobene Bit
	btfsc CLIP_BIT,0		;prfen ob CLIP_BIT = 0
	bsf CLIP_1,0			;Bit ist 1, ersetze das eingeschobene Bit
	clrf CLIP_COUNT			;lsche Register
	movfw CLIP_1			;Speicherregister 1+2 werden ausgezhlt
	call anzahl
	movfw CLIP_2			;
	call anzahl

	movfw CLIP_COUNT		;hole Zhler ins W-Register
	subwf CLIP_COUNT_MAX,w	;subtrahiere max Anzahl
	btfsc STATUS, C			;wenn ergebnis negativ, C=0 nchste Befehl, wenn ergebnis positiv, C=1 berspringe nchsten Befehl
	goto main				;Ergebnis < max, weiter im Programm

lock_amp					;Unterprogramm zum sperren/entsperren des Verstrkers
	call amp_off
lock_amp1					;Reset Taster muss mindestens eine Sekunde gedrckt sein,
	btfsc IN_RESET			;Prfen, ob der Reset Taster gedrckt ist (=> Eingang low)
	goto lock_amp1			;Taster ist nicht gedrckt, zurck zur Schleife
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms Gesamtwartezeit = 1s
	call wait				;Warteschleife
	btfsc IN_RESET			;Prfen, ob der Reset Taster gedrckt ist (=> Eingang low)
	goto lock_amp1			;Taster ist nicht gedrckt, zurck zur Schleife
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife
	btfsc IN_RESET			;Prfen, ob der Reset Taster gedrckt ist (=> Eingang low)
	goto lock_amp1			;Taster ist nicht gedrckt, zurck zur Schleife
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife
	btfsc IN_RESET			;Prfen, ob der Reset Taster gedrckt ist (=> Eingang low)
	goto lock_amp1			;Taster ist nicht gedrckt, zurck zur Schleife
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife		
	btfsc IN_RESET			;Prfen, ob der Reset Taster gedrckt ist (=> Eingang low)
	goto lock_amp1			;Taster ist nicht gedrckt, zurck zur Schleife
	bcf OUT_VOL_M
	btfss IN_RESET			;Prfen, ob der Reset Taster nicht mehr gedrckt ist (=> Eingang high)
	goto $-1				;Taster ist noch gedrckt, Zurck zur Abfrage
	movlw d'20' 			;Wartezeit in ms (max 250) 20ms
	call wait				;Warteschleife	
	btfss IN_RESET			;Nochmaliges Prfen, ob der Reset Taster nicht mehr gedrckt ist (=> Eingang high)
	goto $-1				;Taster ist noch gedrckt, Zurck zur Abfrage	
	goto start				;gehe zu start

learn						;
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife	
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife	
	bcf OUT_LED_RT			;Lsche LED rot
	bcf OUT_VOL_M			;Lsche LED gelb
	clrf IR_NEU				;lsche Statusregister
	bcf INTCON,INTF			;lsche Interruptflag
	bsf INTCON,INTE			;erlaube RB0 interrupts
	bsf INTCON,GIE			;erlaube globale interrupts

	btfss IR_NEU,0			;Abfrage, ob neuer IR-Befehl empfangen wurde
	goto $-1				;kein Befehl, zurck zur Abfrage
	btfss IR_NEU,1			;Abfrage ob Befehl fehlerhaft bzw. kein RC5 Code war
	goto $+3				;Befehl war OK (RC5 Code)
	bsf OUT_LED_RT			;Befehl war fehlerhaft, setze Ausgang fr rote LED
	goto learn				;und zurck zur Schleife
	bsf OUT_VOL_M			;Befehl war OK (RC5 Code), setze Ausgang fr gelbe LED
	btfsc IN_RESET			;prfen, ob Taster zum Speichern des Codes gedrckt ist
	goto learn				;Taster ist nicht gedrckt, zurck zur Schleife
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife	
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife	
	bcf OUT_VOL_M			;Lsche LED gelb
	btfss IN_RESET			;prfen, ob Taster zum Speichern des Codes noch gedrckt ist
	goto $-1				;Taster ist noch gedrckt, zurck zur Prfung

	movfw IR_ADR			;Taster ist nicht mehr gedrckt, Speichern der Adresse und des Befehls im EEPROM
	movwf TEMP				;kopiere die IR-Adresse ins Temp Register
	movlw d'0'				;Schreibe die EEProm Adresse fr die 1. IR-Adresse ins W-Register
	btfss STAT_AMP,2		;Prfen ob das Bit fr die 1. Adresse/Befehl gesetzt ist
	movlw d'2'				;Bit ist nicht gesetzt, Schreibe die EEProm Adresse fr die 2. IR-Adresse ins W-Register
	call writeEE			;Aufruf des Unterprogramms zum Speichern im EEProm 
	movfw IR_BEF			;kopiere den IR-Befehl ins Temp Register
	movwf TEMP				;
	movlw d'1'				;Schreibe die EEProm Adresse fr den 1. IR-Befehl ins W-Register
	btfss STAT_AMP,2		;Prfen ob das Bit fr die 1. Adresse/Befehl gesetzt ist
	movlw d'3'				;Bit ist nicht gesetzt, Schreibe die EEProm Adresse fr den 2. IR-Befehl ins W-Register
	call writeEE			;Aufruf des Unterprogramms zum Speichern im EEProm
	btfss STAT_AMP,2		;Prfen ob das Bit fr die 1. Adresse/Befehl gesetzt ist
	goto $+3
	bcf STAT_AMP,2			;Lsche Statusbit fr 1. Adresse/Befehl, d.h. die 2. Adresse/Befehl wird beim nchsten mal gespeichert 	
	goto learn				;zurck zur Schleife	
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife	
	goto start				;springe zu Start

;===========================
;  Unterprogramme
;===========================
 
wait						;Unterprogramm Warteschleife (max 250 ms)
	movwf   LOOP_1
wait_1
	movlw   d'250'			;die wait_1-Schleife verzgert 1 ms = 250 x 4s
	movwf   LOOP_2 
wait_2
	nop						;die wait_2-Schleife verzgert 4 Takte = 4 s
	decfsz  LOOP_2,F  
	goto    wait_2 
	decfsz  LOOP_1,F 
	goto    wait_1 
	return

anzahl
	movwf TEMP				;der zu analysierende wert wird ins TEMP Register kopiert
	movlw d'8'				;8 bits sollen analysiert werden
	movwf LOOP_1			;Zhler fr die auszuwertenden bits
rotieren
	rlf TEMP,f				;Das Zeichen wird nach links verschoben. Das Carryflag wird an den Ausgang weitergegeben
	btfsc STATUS,C			;Prfen, welchen Wert das Carryflag hat
	incf CLIP_COUNT,f		;Carryflag = 1 -> Zhler um eins erhhen
	decfsz LOOP_1,f			;ein bit wurde bertragen -> Zhler um 1 verringern
	goto rotieren			;nchstes bit bertragen
	return		 

amp_on
	bsf OUT_POWER			;setze Ausgang fr Power On
	bcf OUT_LED_RT			;Lsche rote LED
	bsf STAT_AMP,1			;Setze Statusbit fr On IR/Taster
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms Gesamtwartezeit = 1s
	call wait				;Warteschleife
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife	
	bsf OUT_STBY			;Amp ist freigegeben, setze Ausgang Stand By auf high
	movlw d'100' 			;Wartezeit in ms (max 250) 100ms
	call wait				;Warteschleife
	bsf OUT_MUTE			;setze Ausgang Mute auf high
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife
	return

amp_off						;Unterprogramm fr Verstrker Abschaltung
	bcf OUT_MUTE			;setze Ausgang MUTE auf low
	bsf OUT_LED_RT			;Setze rote LED
	bcf STAT_AMP,1			;Lsche Statusbit fr On IR/Taster
	movlw d'50' 			;Wartezeit in ms (max 250) 50ms
	call wait				;Warteschleife
	bcf OUT_STBY			;setze Ausgang Stand By auf low
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife
	bcf OUT_POWER			;setze Ausgang Power auf low
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call wait				;Warteschleife
	return

ir_onoff
	btfsc STAT_AMP,1
	goto $+5
	call amp_on
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms 
	call wait				;Warteschleife
	return
	call amp_off
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms 
	call wait				;Warteschleife
	return

readEE							;Unterprogramm zum lesen der Zelle deren Adresse in W steht (internes EEPROM)
	bsf STATUS, RP0 			;EEADR liegt in der Bank 1 
 	movwf EEADR 				;schreibe die Adresse in EEADR 
	bsf EECON1, RD 				;EEPROM Leseproze starten 
	movf EEDATA, W 				;Die Daten der EEPROM Zelle nach W kopieren
	bcf STATUS, RP0 			;Bank 0
	return

writeEE							;Unterprogramm zum Schreiben des Wertes in TEMP_13 an die Adresse, die in W steht (internes EEPROM)
	bsf STATUS, RP0				;Bank 1 (EECON1 ist in Bank 1)
	btfsc EECON1, WR			;prfen, ob das EEPROM beschrieben werden kann
	goto $-1					;warten, bis das bit WR von der Hardware zurckgesetzt wurde
	movwf EEADR					;Die Adresse an die die Daten geschrieben werden sollen ist im W-Register
	bcf STATUS, RP0				;Bank 0
	movfw TEMP					;hole zu speichernden Wert ins W-Register
	bsf STATUS, RP0				;Bank 1 (EEADR und EEDATA sind in Bank 1)
	movwf EEDATA				;Wert wird in EEADR geschrieben.
	bsf EECON1, WREN			;erlaubt das Beschreiben des Daten-EEPROM
	bcf INTCON, GIE				;falls Interrupts verwendet werde, mssen diese abgeschaltet werden	
	movlw 0x55					;Um das EEPROM zu beschreiben, mssen die folgenden 5 Befehle
	movwf EECON2				;genau in dieser Reihenfolge aufgerufen werden.
	movlw 0xAA					;
	movwf EECON2				;
	bsf EECON1, WR				;Jetzt beginnt der eigentliche Schreibvorgang.
	bsf INTCON, GIE				;Die Interrupts knnen jetzt wieder freigegeben werden.
	bcf EECON1, WREN			;Das Beschreiben des EEPROM verbieten.
	bcf STATUS, RP0				;Bank 0
	return						;Zurck ins Hauptprogramm

;===============================================================
;  Unterprogram zur Auswertung des RC5 Codes eines IR-Empfngers
;===============================================================

ir_startbit
	clrf IR_BIT				;lsche IR-Bit
	clrf IR_TEMP			;Lsche IR_TEMP Register, dieses wird als temporres Register zur Zwischenspeicherung des IR-Bits verwendet
							;2.Hlfte
	movlw d'1'				;440us Zeitverzgerung bis zur ersten Abtastung = 1 x 256us, Rest = 184us
	movwf LOOP_1us			;
	movlw d'184'			;Rest = 184us 
	call wait_us			;
	btfsc IN_IR				;prfen ob IR-Eingangssignal low ist
	goto fehler				;

startbit2
	movlw d'1'				;1 Bit soll ausgewertet werden
	movwf COUNTER
	call ir_routine			;Aufruf des Unterprogramms
	btfsc IR_BIT,2			;prfen ob das Fehlerbit gesetzt ist
	goto fehler				;Fehlerbit ist gesetzt => weiter bei Fehler
	movfw IR_TEMP			;Sichern des Bits im Register IR_START2
	movwf IR_START2	

steuerbit
	movlw d'1'				;1 Bit soll ausgewertet werden
	movwf COUNTER
	call ir_routine			;Aufruf des Unterprogramms
	btfsc IR_BIT,2			;prfen ob das Fehlerbit gesetzt ist
	goto fehler				;Fehlerbit ist gesetzt => weiter bei Fehler
	movfw IR_TEMP			;Sichern des Bits im Register IR_STRG
	movwf IR_STRG

adressbits
	movlw d'5'				;5 Bit soll ausgewertet werden
	movwf COUNTER
	call ir_routine			;Aufruf des Unterprogramms
	btfsc IR_BIT,2			;prfen ob das Fehlerbit gesetzt ist
	goto fehler				;Fehlerbit ist gesetzt => weiter bei Fehler
	movfw IR_TEMP			;Sichern des Bits im Register IR_ADR
	movwf IR_ADR

befehlsbits
	movlw d'6'				;6 Bit soll ausgewertet werden
	movwf COUNTER
	call ir_routine			;Aufruf des Unterprogramms
	btfsc IR_BIT,2			;prfen ob das Fehlerbit gesetzt ist
	goto fehler				;Fehlerbit ist gesetzt => weiter bei Fehler
	movfw IR_TEMP			;Sichern des Bits im Register IR_BEF
	movwf IR_BEF

	movlw d'3'				;Zeitverzgerung von 1/2 Bit damit die 2.Hlfte des letzten Bits nicht doppelt gezhlt wird
	movwf LOOP_1us			;875us = 3 x 256us, Rest = 107us
	movlw d'107'			;Rest = 107us 
	call wait_us
	
	bsf IR_NEU,0			;setze Statusbit fr neuen IR-Befehl
	return					;Rcksprung aus dem Unterprogramm

fehler
	movlw d'30'				;Wartezeit von 30ms nach einem Fehler
	movwf LOOP_1us			;
	call wait 		
	movlw 0xFF				;Bei Fehler werden die Adress- und Befehlsregister mit Wert 255 beschrieben
	movwf IR_ADR
	movwf IR_BEF
	bsf IR_NEU,0			;setze Statusbit fr neuen IR-Befehl
	bsf IR_NEU,1			;setze Statusbit fr Fehler
	return					;Rcksprung aus dem Unterprogramm

;==================================
;  Unterprogramme fr IR-Auswertung
;==================================

ir_routine
	clrf IR_TEMP			;Lsche IR_TEMP Register, dieses wird als temporres Register zur Zwischenspeicherung des IR-Bits verwendet
ir_rout_loop				;diese Schleife wird ausgefhrt bis COUNTER = 0
	clrf IR_BIT				;lsche IR-Bit Register
	rlf IR_TEMP				;Register wird um 1 nach links verschoben
							;Ermittlung des IR Signalpegels fr die 1.Hlfte des Bits
	movlw d'3'				;875us Zeitverzgerung bis zur ersten Abtastung = 3 x 256us, Rest = 107us
	movwf LOOP_1us			;
	movlw d'107'			;Rest = 107us 
	call wait_us			;
	btfss IN_IR				;prfen ob IR-Eingangssignal high ist
	bsf IR_BIT,0			;wenn ja, berspringe diesen Befehl
	btfsc IN_IR				;prfen ob IR-Eingangssignal low ist
	bcf IR_BIT,0			;wenn ja, berspringe diesen Befehl
							;2.Hlfte							
	movlw d'3'				;875us Zeitverzgerung bis zur zweiten Abtastung = 3 x 256us, Rest = 107us
	movwf LOOP_1us			;
	movlw d'107'			;Rest = 107us 
	call wait_us
	btfss IN_IR				;prfen ob IR-Eingangssignal high ist
	bsf IR_BIT,1			;wenn ja, berspringe diesen Befehl
	btfsc IN_IR				;prfen ob IR-Eingangssignal low ist
	bcf IR_BIT,1			;wenn ja, berspringe diesen Befehl
							;Auswertung
	movlw d'1'				;schreibe Wert 1 ins W-Register
	subwf IR_BIT,w			;subtrahiere Register IR_BIT (= 1 wenn Bit ist 0, = 2 wenn Bit ist 1)
	btfss STATUS,Z			;wenn Ergebnis = 0 , berspringe nchsten Befehl
	goto $+3				;Ergebnis ist nicht 0, weiter bei nchster Prfung
	bcf IR_TEMP,0			;Ergebnis ist 0, setze Bit 0 auf 0
	goto ir_loop_test		;weiter bei ir_loop_test
	movlw d'2'				;schreibe Wert 2 ins W-Register
	subwf IR_BIT,w			;subtrahiere Register IR_BIT (= 1 wenn Bit ist 0, = 2 wenn Bit ist 1)
	btfss STATUS,Z			;wenn Ergebnis = 0 , berspringe nchsten Befehl
	goto $+3				;Ergebnis ist nicht 0 = es ist ein Fehler aufgetreten
	bsf IR_TEMP,0			;Ergebnis ist 0, setze Bit 0 auf 1
	goto ir_loop_test		;weiter bei ir_loop_test
	bsf IR_BIT,2			;setze das Fehler Bit und springe zum Ende des Unterprogramms
	goto ir_rout_end

ir_loop_test
	decfsz COUNTER			;reduziere Zhler um 1 und prfe ob auf Null runtergezhlt
	goto ir_rout_loop		;Zhler ist nicht 0 ,zurck zur Schleife

ir_rout_end
	return

wait_us						;Mikrosekunden Timer mit TMR0
	clrf LOOP_2us			;
	subwf LOOP_2us			; 
	movlw d'25'				;der erste loop wird um 25us verkrzt, da die Abarbeitung des Unterprogramms auch Zeit bentigt
	movwf TMR0				;
	bcf INTCON,T0IF
wait_us1
	btfss INTCON,T0IF
	goto wait_us1
	bcf INTCON,T0IF
	decfsz LOOP_1us
	goto wait_us1
	movfw LOOP_2us
	movwf TMR0
	btfss INTCON,T0IF
	goto $-1
	return

	END                			; Ende des Programms


